
import React, { Component } from 'react';
import { createStore, applyMiddleware } from 'redux';
import ReactDOM from 'react-dom';
import { Provider, connect } from 'react-redux';
import createLogger from 'redux-logger';

import { Field, Control, Form, Errors, combineForms, actions } from 'react-redux-form';

import thunk from 'redux-thunk';

const initialUserState = {
  username: '',
  email: '',
  phones: []
};

const MyState = {
  username: 'Maxim',
  'email': 'test@gmail.com',
};

const store = createStore(combineForms({
  user: initialUserState,
}), applyMiddleware(thunk, createLogger()));

const required = (val) => val && val.length;
const maxLength = (len) => (val) => val.length <= len;
const isNumber = (val) => !isNaN(Number(val));

class LoadModelButton extends Component {
  render () {
    console.log(this.props.dispatch);
    return (<button>Upload</button>);
  }
};

class UserForm extends Component {

  componentDidMount () {
    setTimeout(()=> {
        this.props.dispatch(actions.change('user', MyState));
      }, 1000);
  }
    render() {

    return (
      <Form model="user" onSubmit={v => console.log(v)}>
      <div className="field">
        <label>Username:</label>
        <Field
        model=".username"
        validators={{
          required,
          maxLength: maxLength(15)
        }}
        >
        <input type="text"/>
        </Field>
        <Errors
        className="errors"
        model="user.username"
        messages={{
          required: 'Required',
          maxLength: 'Must be 15 characters or less',
        }}
        />
      </div>

      <div className="field">
        <label>Email:</label>
        <Field
        model=".email"
        validators={{
          required,
          validEmail: (val) => /^[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,4}$/i.test(val),
        }}
        >
        <input type="text" />
        </Field>
        <Errors
        className="errors"
        model="user.email"
        messages={{
          required: 'Required',
          validEmail: 'Invalid email address',
        }}
        />
      </div>

      <div className="field">
        <label>Phones</label>
        <Field
        model=".phones[0]"
        validators={{
          required,
          validEmail: (val) => /^[A-Z0-9._%+-]+@[A-Z0-9.-]+\.[A-Z]{2,4}$/i.test(val),
        }}
        >
        <input type="text" />
        </Field>
        <Errors
        className="errors"
        model="user.phones[0]"
        messages={{
          required: 'Required',
          validEmail: 'Invalid phone number',
        }}
        />
      </div>

      
      <div className="field">
        <label>Age:</label>
        <Field
        model=".age"
        validators={{
          required,
          isNumber,
          minAge: (val) => val >= 18,
        }}
        >
        <input type="text" />
        </Field>
        <Errors
        className="errors"
        model="user.age"
        messages={{
          required: 'Required',
          isNumber: 'Must be a number',
          minAge: 'Sorry, you must be at least 18 years old',
        }}
        />
      </div>
      
      <button type="submit">
        Submit
      </button>

      <button onClick={()=> this.props.dispatch(actions.change('user', MyState))}>
        Load
      </button>
      
      <Control.reset model="user" type="reset">
        Reset
      </Control.reset>
      </Form>
    )
  }
}

class App extends React.Component {
  render() {
  return (
    <Provider store={store}>
    <UserForm />
    </Provider>
  );
  }
}

UserForm = connect((state)=> {

  return state;
}, (dispatch)=>{
  return {dispatch};
})(UserForm);

ReactDOM.render(<App />, document.getElementById('app'));
